@@ -5,6 +5,7 @@
 #//   Copyright 2007-2011 Cadence Design Systems, Inc.
 #//   Copyright 2010-2011 Synopsys, Inc.
 #//   Copyright 2013      NVIDIA Corporation
+#//   Copyright 2019-2021 Tuomas Poikela
 #//   All Rights Reserved Worldwide
 #//
 #//   Licensed under the Apache License, Version 2.0 (the
@@ -29,8 +30,11 @@ from .uvm_debug import uvm_debug
 from .uvm_globals import *
 from .uvm_object_globals import (UVM_RAISED, UVM_DROPPED, UVM_ALL_DROPPED)
 from .sv import sv
-from ..macros import uvm_error
+from ..macros import (uvm_error, uvm_info, uvm_do_callbacks,
+    uvm_do_callbacks_async)
 from typing import List, Optional, Dict, Any
+from .uvm_pool import UVMPool
+from .uvm_callback import UVMCallback
 
 UVM_USE_PROCESS_CONTAINER = 1
 
@@ -52,6 +56,25 @@ class UVMObjectionEvents():
 ObjContextDict = Dict[Any, 'UVMObjectionContextObject']
 ObjContextList = List['UVMObjectionContextObject']
 
+
+def get_name_depth(curr_obj_name: str) -> int:
+    """ Returns the hier depth of the name """
+    depth = 0
+    for i in curr_obj_name:
+        if i == ".":
+            depth += 1
+    return depth
+
+
+def get_leaf_name(curr_obj_name: str) -> str:
+    name = curr_obj_name
+    rr = range(len(curr_obj_name)-1, -1, -1)
+    for i in rr:
+        if curr_obj_name[i] == ".":
+            name = curr_obj_name[i+1:len(curr_obj_name)]
+            break
+    return name
+
 #//------------------------------------------------------------------------------
 #// Title: Objection Mechanism
 #//------------------------------------------------------------------------------
@@ -59,6 +82,64 @@ ObjContextList = List['UVMObjectionContextObject']
 #// functionality, which is based on <uvm_objection>.
 #//------------------------------------------------------------------------------
 
+#//------------------------------------------------------------------------------
+#//
+#// Class: uvm_objection_callback
+#//
+#//------------------------------------------------------------------------------
+#// The uvm_objection is the callback type that defines the callback
+#// implementations for an objection callback. A user uses the callback
+#// type uvm_objection_cbs_t to add callbacks to specific objections.
+#//
+#// For example:
+#//
+#//| class my_objection_cb extends uvm_objection_callback
+#//|   function new(string name)
+#//|     super.new(name)
+#//|   endfunction
+#//|
+#//|   virtual function void raised (uvm_objection objection, uvm_object obj,
+#//|       uvm_object source_obj, string description, int count)
+#//|       `uvm_info("RAISED","%0t: Objection %s: Raised for %s", $time, objection.get_name(),
+#//|       obj.get_full_name())
+#//|   endfunction
+#//| endclass
+#//| ...
+#//| initial begin
+#//|   my_objection_cb cb = new("cb")
+#//|   uvm_objection_cbs_t::add(null, cb); //typewide callback
+#//| end
+
+
+class UVMObjectionCallback(UVMCallback):
+    def __init__(self, name):
+        super().__init__(name)
+#  // Function: raised
+#  //
+#  // Objection raised callback function. Called by <uvm_objection::raised>.
+#
+#  virtual function void raised (uvm_objection objection, uvm_object obj,
+#      uvm_object source_obj, string description, int count)
+#  endfunction
+#
+#  // Function: dropped
+#  //
+#  // Objection dropped callback function. Called by <uvm_objection::dropped>.
+#
+#  virtual function void dropped (uvm_objection objection, uvm_object obj,
+#      uvm_object source_obj, string description, int count)
+#  endfunction
+#
+#  // Function: all_dropped
+#  //
+#  // Objection all_dropped callback function. Called by <uvm_objection::all_dropped>.
+#
+#  virtual task all_dropped (uvm_objection objection, uvm_object obj,
+#      uvm_object source_obj, string description, int count)
+#  endtask
+#
+#endclass
+
 #//------------------------------------------------------------------------------
 #//
 #// Class: uvm_objection
@@ -187,17 +268,18 @@ class UVMObjection(UVMReportObject):
     #  // 0 turns tracing off. A trace mode of 1 turns tracing on.
     #  // The return value is the mode prior to being reset.
     #
-    #   function bit trace_mode (int mode=-1)
-    #    trace_mode = self.m_trace_mode
-    #    if(mode == 0) self.m_trace_mode = 0
-    #    else if(mode == 1) self.m_trace_mode = 1
-    #   endfunction
-    #
+    def trace_mode(self, mode=-1):
+        trace_mode = self.m_trace_mode
+        if mode == 0:
+            self.m_trace_mode = 0
+        elif mode == 1:
+            self.m_trace_mode = 1
+        return trace_mode
+
     #  // Function- m_report
     #  //
     #  // Internal method for reporting count updates
     #
-    #def m_report(uvm_object obj, uvm_object source_obj, string description, int count, string action)
     def m_report(self, obj, source_obj, description: str, count: int, action: str) -> None:
         _count = 0
         if obj in self.m_source_count:
@@ -242,10 +324,10 @@ class UVMObjection(UVMReportObject):
         if name == "":
             name = "uvm_top"
         act_type = "subtracted"
-        if action=="raised":
+        if action == "raised":
             act_type = "added"
         act_dir = "from"
-        if action=="raised":
+        if action == "raised":
             act_dir = "to"
         uvm_report_info("OBJTN_TRC",
             sv.sformatf("Object %0s %0s %0d objection(s) %0s its total (%s from"
@@ -271,7 +353,7 @@ class UVMObjection(UVMReportObject):
         if obj is None:
             obj = self.m_top
         return obj
-    #  endfunction
+
 
     #  // Function- m_propagate
     #  //
@@ -287,13 +369,13 @@ class UVMObjection(UVMReportObject):
     #  // raise : indicator of whether the objection is being raised or lowered. A
     #  //   1 indicates the objection is being raised.
     #
-    def m_propagate (self, obj, source_obj, description, count, raise_, in_top_thread):
+    def m_propagate(self, obj, source_obj, description, count, raise_, in_top_thread):
         if obj is not None and obj != self.m_top:
-            obj = self.m_get_parent(obj)
-            if(raise_):
-                self.m_raise(obj, source_obj, description, count)
+            parent_obj = self.m_get_parent(obj)
+            if raise_:
+                self.m_raise(parent_obj, source_obj, description, count)
             else:
-                self.m_drop(obj, source_obj, description, count, in_top_thread)
+                self.m_drop(parent_obj, source_obj, description, count, in_top_thread)
 
     #  // Group: Objection Control
     #
@@ -348,13 +430,12 @@ class UVMObjection(UVMReportObject):
     #
     #     self.m_prop_mode = prop_mode
     #  endfunction : set_propagate_mode
-    #
+
     #  // Function: get_propagate_mode
     #  // Returns the propagation mode for this objection.
-    #  function bit get_propagate_mode()
-    #     return self.m_prop_mode
-    #  endfunction : get_propagate_mode
-    #
+    def get_propagate_mode(self) -> int:
+        return self.m_prop_mode
+
     #  // Function: raise_objection
     #  //
     #  // Raises the number of objections for the source ~object~ by ~count~, which
@@ -372,7 +453,7 @@ class UVMObjection(UVMReportObject):
     #  //   <uvm_component::raised> method for all of the components up the
     #  //   hierarchy.
     #  //
-    def raise_objection (self, obj=None, description="", count=1):
+    def raise_objection(self, obj=None, description="", count=1):
         if obj is None:
             obj = self.m_top
         self.m_cleared = 0
@@ -409,11 +490,12 @@ class UVMObjection(UVMReportObject):
         # First go through the scheduled list
         idx = 0
         m_scheduled_list = UVMObjection.m_scheduled_list
-        while idx < len(m_scheduled_list):
-            if ((m_scheduled_list[idx].obj == obj) and
-                    (m_scheduled_list[idx].objection == self)):
+        # while idx < len(m_scheduled_list):
+        for entry in m_scheduled_list:
+            if ((entry.obj == obj) and
+                    (entry.objection == self)):
                 # Caught it before the drain was forked
-                ctxt = m_scheduled_list[idx]
+                ctxt = entry
                 del m_scheduled_list[idx]
                 break
             idx += 1
@@ -422,7 +504,7 @@ class UVMObjection(UVMReportObject):
         if ctxt is None:
             idx = 0
             while idx < len(self.m_forked_list):
-                if (self.m_forked_list[idx].obj == obj):
+                if self.m_forked_list[idx].obj == obj:
                     # Caught it after the drain was forked,
                     # but before the fork started
                     ctxt = self.m_forked_list[idx]
@@ -459,7 +541,6 @@ class UVMObjection(UVMReportObject):
 
         else:
             # Otherwise we need to determine what exactly happened
-            diff_count = 0
 
             # Determine the diff count, if it's positive, then we're
             # looking at a 'raise' total, if it's negative, then
@@ -552,6 +633,7 @@ class UVMObjection(UVMReportObject):
         if obj is None:
             obj = self.m_top
         uvm_debug(self, 'drop_objection', obj.get_name() + " Starting to drop objection")
+
         self.m_drop(obj, obj, description, count, 0)
 
     #  // Function- m_drop
@@ -559,28 +641,30 @@ class UVMObjection(UVMReportObject):
     def m_drop(self, obj, source_obj, description="", count=1,
             in_top_thread=0):
         # Ignore drops if the count is 0
-        if (count == 0):
+        if count == 0:
             return
 
         if obj not in self.m_total_count or (count > self.m_total_count[obj]):
             if self.m_cleared:
                 return
-            uvm_report_fatal("OBJTN_ZERO", ("Object \"" + obj.get_full_name()
-              + "\" attempted to drop objection '" + self.get_name() + "' count below zero"))
+            uvm_report_fatal("OBJTN_ZERO", ("Object '" + obj.get_full_name()
+              + "' attempted to drop total objection '" + self.get_name() + "' count below zero. " +
+              "Description: '" + description + "' source_obj: " +
+              source_obj.get_name()))
             return
 
         if obj == source_obj:
             if obj not in self.m_source_count or (count > self.m_source_count[obj]):
-                if (self.m_cleared):
+                if self.m_cleared:
                     return
                 uvm_report_fatal("OBJTN_ZERO", ("Object \"" + obj.get_full_name()
-                  + "\" attempted to drop objection '" + self.get_name() + "' count below zero"))
+                  + "\" attempted to drop source objection '" + self.get_name() + "' count below zero"))
                 return
             self.m_source_count[obj] -= count
 
         self.m_total_count[obj] -= count
 
-        if (self.m_trace_mode):
+        if self.m_trace_mode:
             self.m_report(obj,source_obj,description,count,"dropped")
 
         self.dropped(obj, source_obj, description, count)
@@ -589,7 +673,7 @@ class UVMObjection(UVMReportObject):
         if self.m_total_count[obj] != 0:
             if not self.m_prop_mode and obj != self.m_top:
                 self.m_drop(self.m_top,source_obj,description, count, in_top_thread)
-            elif (obj != self.m_top):
+            elif obj != self.m_top:
                 self.m_propagate(obj, source_obj, description, count, 0, in_top_thread)
         else:
             ctxt = None  # uvm_objection_context_object
@@ -617,19 +701,18 @@ class UVMObjection(UVMReportObject):
             UVMObjection.m_scheduled_list.append(ctxt)
             UVMObjection.m_scheduled_list_not_empty_event.set()
         #end // else: !if(self.m_total_count[obj] != 0)
-        #
-        #  endfunction
 
-    #  // Function: clear
-    #  //
-    #  // Immediately clears the objection state. All counts are cleared and the
-    #  // any processes waiting on a call to wait_for(UVM_ALL_DROPPED, uvm_top)
-    #  // are released.
-    #  //
-    #  // The caller, if a uvm_object-based object, should pass its 'this' handle
-    #  // to the ~obj~ argument to document who cleared the objection.
-    #  // Any drain_times set by the user are not affected.
-    #  //
+
+    # Function: clear
+    #
+    # Immediately clears the objection state. All counts are cleared and the
+    # any processes waiting on a call to wait_for(UVM_ALL_DROPPED, uvm_top)
+    # are released.
+    #
+    # The caller, if a uvm_object-based object, should pass its 'this' handle
+    # to the ~obj~ argument to document who cleared the objection.
+    # Any drain_times set by the user are not affected.
+    #
     def clear(self, obj=None):
         name = ""
         ctxt = None  # uvm_objection_context_object
@@ -638,13 +721,11 @@ class UVMObjection(UVMReportObject):
         uvm_debug(self, 'clear', 'START')
         if obj is None:
             obj = self.m_top
-        name = obj.get_full_name()
-        if (name == ""):
-            name = "uvm_top"
-        else:
-            name = obj.get_full_name()
 
-        uvm_debug(self, 'clear', 'KKK MMM')
+        name = obj.get_full_name()
+        if name == "":
+            name = "uvm_top"
+
         if (self.m_top_all_dropped is False and
                 self.get_objection_total(self.m_top) > 0):
             uvm_report_warning("OBJTN_CLEAR",("Object '" + name
@@ -653,7 +734,6 @@ class UVMObjection(UVMReportObject):
         self.m_source_count = {}
         self.m_total_count = {}
 
-        uvm_debug(self, 'clear', 'KKK MMM')
         # Remove any scheduled drains from the static queue
         idx = 0
         while (idx < len(UVMObjection.m_scheduled_list)):
@@ -695,44 +775,44 @@ class UVMObjection(UVMReportObject):
     #  // -------------------------
     #  // background process; when non
 
-    async def m_execute_scheduled_forks(self):
+    @classmethod
+    async def m_execute_scheduled_forks(cls):
         while True:
             #wait(UVMObjection.m_scheduled_list.size() != 0)
-            uvm_debug(self, 'm_execute_scheduled_forks', 'waiting list to not be empty')
+            uvm_debug(cls, 'm_execute_scheduled_forks', 'waiting list to not be empty')
             await UVMObjection.m_scheduled_list_not_empty_event.wait()
             UVMObjection.m_scheduled_list_not_empty_event.clear()
 
             if len(UVMObjection.m_scheduled_list) != 0:
-                # c = None  # uvm_objection_context_object
-                # o = None  # uvm_objection
                 # Save off the context before the fork
-                c = UVMObjection.m_scheduled_list[0]
-                UVMObjection.m_scheduled_list.remove(c)
+                c = UVMObjection.m_scheduled_list.pop(0)
+
                 # A re-raise can use this to figure out props (if any)
                 objection = c.objection
-                if objection is not None:
-                    objection.m_scheduled_contexts[c.obj] = c
-                    # The fork below pulls out from the forked list
-                    objection.m_forked_list.append(c)
-                    # The fork will guard the m_forked_drain call, but
-                    # a re-raise can kill self.m_forked_list contexts in the delta
-                    # before the fork executes.
-                    pproc = cocotb.fork(UVMObjection().m_execute_scheduled_forks_fork_join_none(c))
-                else:
-                    uvm_error("UVMObjection", "Null objection in objection context")
-        #endtask
+                #rm if objection is not None:
+                objection.m_scheduled_contexts[c.obj] = c
+                # The fork below pulls out from the forked list
+                objection.m_forked_list.append(c)
+                # The fork will guard the m_forked_drain call, but
+                # a re-raise can kill self.m_forked_list contexts in the delta
+                # before the fork executes.
+                pproc = cocotb.fork(cls.m_execute_scheduled_forks_fork_join_none(c))
+                #else:
+                #    uvm_error("UVMObjection", "Null objection in objection context")
 
 
-    async def m_execute_scheduled_forks_fork_join_none(self,
+    @classmethod
+    async def m_execute_scheduled_forks_fork_join_none(cls,
             c: 'UVMObjectionContextObject'):
         objection = c.objection  # automatic uvm_objection
         # Check to maike sure re-raise didn't empty the fifo
-        uvm_debug(self, 'm_execute_scheduled_forks_fork_join_none', 'check list len')
+        uvm_debug(cls, 'm_execute_scheduled_forks_fork_join_none', 'check list len')
         if len(objection.m_forked_list) > 0:
             #uvm_objection_context_object ctxt
             ctxt = objection.m_forked_list.pop(0)
             # Clear it out of scheduled
             del objection.m_scheduled_contexts[ctxt.obj]
+
             # Move it in to forked (so re-raise can figure out props)
             objection.m_forked_contexts[ctxt.obj] = ctxt
             # Save off our process handle, so a re-raise can kill it...
@@ -752,12 +832,15 @@ class UVMObjection(UVMReportObject):
                 del objection.m_drain_proc[ctxt.obj]
             else:
                 pass
+
+            # tpoikela: Added check since ctxt.obj becomes None
+            #if ctxt.obj in objection.m_forked_contexts:
             del objection.m_forked_contexts[ctxt.obj]
             # Clear out the context object (prevent memory leaks)
             ctxt.clear()
             # Save the context in the pool for later reuse
             UVMObjection.m_context_pool.append(ctxt)
-        await uvm_zero_delay()
+        #rm await uvm_zero_delay()
 
     #  // m_forked_drain
     #  // -------------
@@ -769,7 +852,7 @@ class UVMObjection(UVMReportObject):
 
     async def m_forked_drain(self, obj, source_obj, description="", count=1,
             in_top_thread=0):
-        diff_count = 0
+        #rm diff_count = 0
 
         if obj in self.m_drain_time:
             # pass
@@ -779,7 +862,7 @@ class UVMObjection(UVMReportObject):
         if self.m_trace_mode:
             self.m_report(obj,source_obj,description,count,"all_dropped")
 
-        self.all_dropped(obj,source_obj,description, count)
+        await self.all_dropped(obj,source_obj,description, count)
 
         # wait for all_dropped cbs to complete
         await uvm_zero_delay()
@@ -797,17 +880,16 @@ class UVMObjection(UVMReportObject):
             self.m_drop(self.m_top,source_obj,description, count, 1)
         elif obj != self.m_top:
             self.m_propagate(obj, source_obj, description, count, 0, 1)
-    #  endtask
 
     #  // m_init_objections
     #  // -----------------
     #
     #  // Forks off the single background process
-
-    async def m_init_objections(self):
+    @classmethod
+    async def m_init_objections(cls):
         #uvm_debug(cls, 'm_init_objections', "Forking m_execute_scheduled_forks")
-        pproc = cocotb.fork(UVMObjection().m_execute_scheduled_forks())
-        await uvm_zero_delay()
+        pproc = cocotb.fork(cls.m_execute_scheduled_forks())
+        #await uvm_zero_delay()
 
     #  // Function: set_drain_time
     #  //
@@ -840,6 +922,7 @@ class UVMObjection(UVMReportObject):
         if hasattr(obj, 'raised'):
             obj.raised(self, source_obj, description, count)
         # TODO `uvm_do_callbacks(uvm_objection,uvm_objection_callback,raised(this,obj,source_obj,description,count))
+        uvm_do_callbacks(self, UVMObjectionCallback, 'raised', self,obj,source_obj,description,count)
         if obj in self.m_events:
             self.m_events[obj].raised.set()
 
@@ -857,9 +940,10 @@ class UVMObjection(UVMReportObject):
         #if($cast(comp,obj))
         comp.dropped(self, source_obj, description, count)
         # TODO `uvm_do_callbacks(uvm_objection,uvm_objection_callback,dropped(this,obj,source_obj,description,count))
+        uvm_do_callbacks(self,UVMObjectionCallback,'dropped', self,obj,source_obj,description,count)
         if obj in self.m_events:
             self.m_events[obj].dropped.set()
-    #  endfunction
+
 
     #  // Function: all_dropped
     #  //
@@ -871,11 +955,11 @@ class UVMObjection(UVMReportObject):
     #                            uvm_object source_obj,
     #                            string description,
     #                            int count)
-    def all_dropped(self, obj, source_obj, description, count):
+    async def all_dropped(self, obj, source_obj, description, count):
         comp = obj
-        #if($cast(comp,obj))
-        comp.all_dropped(self, source_obj, description, count)
+        await comp.all_dropped(self, source_obj, description, count)
         # TODO `uvm_do_callbacks(uvm_objection,uvm_objection_callback,all_dropped(this,obj,source_obj,description,count))
+        await uvm_do_callbacks_async(self, UVMObjectionCallback, 'all_dropped', self,obj, source_obj, description, count)
         if obj in self.m_events:
             self.m_events[obj].all_dropped.set()
         if obj == self.m_top:
@@ -941,15 +1025,13 @@ class UVMObjection(UVMReportObject):
     #  //
     #  // Returns the current number of objections raised by the given ~object~.
     #
-    #  function int get_objection_count (uvm_object obj=null)
-    #    if (obj==null)
-    #      obj = self.m_top
-    #
-    #    if (!self.m_source_count.exists(obj))
-    #      return 0
-    #    return self.m_source_count[obj]
-    #  endfunction
+    def get_objection_count(self, obj=None):
+        if obj is None:
+            obj = self.m_top
 
+        if obj not in self.m_source_count:
+            return 0
+        return self.m_source_count[obj]
 
     #  // Function: get_objection_total
     #  //
@@ -968,7 +1050,7 @@ class UVMObjection(UVMReportObject):
             cnt = self.m_total_count[obj]
             uvm_debug(self, 'get_objection_total', 'Returning cnt ' + str(cnt))
             return self.m_total_count[obj]
-        #endfunction
+
 
     #  // Function: get_drain_time
     #  //
@@ -987,86 +1069,77 @@ class UVMObjection(UVMReportObject):
 
     #  // m_display_objections
     #
-    #  protected function string m_display_objections(uvm_object obj=null, bit show_header=1)
-    #
-    #    static string blank="                                                                                   "
-    #
-    #    string s
-    #    int total
-    #    uvm_object list[string]
-    #    uvm_object curr_obj
-    #    int depth
-    #    string name
-    #    string this_obj_name
-    #    string curr_obj_name
-    #
-    #    foreach (self.m_total_count[o]) begin
-    #      uvm_object theobj = o
-    #      if ( self.m_total_count[o] > 0)
-    #        list[theobj.get_full_name()] = theobj
-    #    end
-    #
-    #    if (obj==null)
-    #      obj = self.m_top
-    #
-    #    total = get_objection_total(obj)
-    #
-    #    s = $sformatf("The total objection count is %0d\n",total)
-    #
-    #    if (total == 0)
-    #      return s
-    #
-    #    s = {s,"---------------------------------------------------------\n"}
-    #    s = {s,"Source  Total   \n"}
-    #    s = {s,"Count   Count   Object\n"}
-    #    s = {s,"---------------------------------------------------------\n"}
-    #
-    #
-    #    this_obj_name = obj.get_full_name()
-    #    curr_obj_name = this_obj_name
-    #
-    #    do begin
-    #
-    #      curr_obj = list[curr_obj_name]
-    #
-    #      // determine depth
-    #      depth=0
-    #      foreach (curr_obj_name[i])
-    #        if (curr_obj_name[i] == ".")
-    #          depth++
-    #
-    #      // determine leaf name
-    #      name = curr_obj_name
-    #      for (int i=curr_obj_name.len()-1;i >= 0; i--)
-    #        if (curr_obj_name[i] == ".") begin
-    #           name = curr_obj_name.substr(i+1,curr_obj_name.len()-1)
-    #           break
-    #        end
-    #      if (curr_obj_name == "")
-    #        name = "uvm_top"
-    #      else
-    #        depth++
-    #
-    #      // print it
-    #      s = {s, $sformatf("%-6d  %-6d %s%s\n",
-    #         self.m_source_count.exists(curr_obj) ? self.m_source_count[curr_obj] : 0,
-    #         self.m_total_count.exists(curr_obj) ? self.m_total_count[curr_obj] : 0,
-    #         blank.substr(0,2*depth), name)}
-    #
-    #    end while (list.next(curr_obj_name) &&
-    #        curr_obj_name.substr(0,this_obj_name.len()-1) == this_obj_name)
-    #
-    #    s = {s,"---------------------------------------------------------\n"}
-    #
-    #    return s
-    #
-    #  endfunction
-    #
-    #
-    #  function string convert2string()
-    #    return m_display_objections(self.m_top,1)
-    #  endfunction
-    #
+    def m_display_objections(self, obj=None, show_header=1):
+        blank="                                                                                   "
+        s = ""
+        total = 0
+        lst = UVMPool()
+        curr_obj = None
+        depth = 0
+        name = ""
+        this_obj_name = ""
+        curr_obj_name = ""
+
+        for o in self.m_total_count:
+            theobj = o
+            if self.m_total_count[o] > 0:
+                lst[theobj.get_full_name()] = theobj
+
+        if obj is None:
+            obj = self.m_top
+
+        total = self.get_objection_total(obj)
+        s = sv.sformatf("The total objection count is %0d\n",total)
+
+        if total == 0:
+            return s
+
+        s = s + "---------------------------------------------------------\n"
+        s = s + "Source  Total   \n"
+        s = s + "Count   Count   Object\n"
+        s = s + "---------------------------------------------------------\n"
+
+        this_obj_name = obj.get_full_name()
+        curr_obj_name = this_obj_name
+
+        while True:
+            curr_obj = lst[curr_obj_name]
+
+            # determine depth, count dots "." in the name
+            depth = get_name_depth(curr_obj_name)
+
+            # determine leaf name
+            name = get_leaf_name(curr_obj_name)
+
+            if curr_obj_name == "":
+                name = "uvm_top"
+            else:
+                depth += 1
+
+            # print it
+            src_count = 0
+            if curr_obj in self.m_source_count:
+                src_count = self.m_source_count[curr_obj]
+            total_count = 0
+            if curr_obj in self.m_total_count:
+                total_count = self.m_total_count[curr_obj]
+            s += sv.sformatf("%d       %d       %s'%s'\n", src_count, total_count,
+                    blank[0:2*depth], name)
+
+            if not ((lst.has_next() and
+                    curr_obj_name[0:len(this_obj_name)] == this_obj_name)):
+                break
+            curr_obj_name = lst.next()
+
+        s = s + "---------------------------------------------------------\n"
+
+        return s
+
+
+
+    def convert2string(self):
+        return self.m_display_objections(self.m_top,1)
+
     #
     #  // Function: display_objections
     #  //
@@ -1074,12 +1147,10 @@ class UVMObjection(UVMReportObject):
     #  // not specified or ~null~, the implicit top-level component, <uvm_root>, is
     #  // chosen. The ~show_header~ argument allows control of whether a header is
     #  // output.
-    #
-    #  function void display_objections(uvm_object obj=null, bit show_header=1)
-    #	string m = m_display_objections(obj,show_header)
-    #    `uvm_info("UVM/OBJ/DISPLAY",m,UVM_NONE)
-    #  endfunction
-    #
+    def display_objections(self, obj=None, show_header=1):
+        m = self.m_display_objections(obj,show_header)
+        uvm_info("UVM/OBJ/DISPLAY", m, UVM_NONE)
+
     #
     #  // Below is all of the basic data stuff that is needed for a uvm_object
     #  // for factory registration, printing, comparing, etc.
@@ -1097,17 +1168,15 @@ class UVMObjection(UVMReportObject):
     #  virtual function string get_type_name ()
     #    return "uvm_objection"
     #  endfunction
-    #
-    #  function void do_copy (uvm_object rhs)
-    #    uvm_objection _rhs
-    #    $cast(_rhs, rhs)
-    #    self.m_source_count = _rhs.m_source_count
-    #    self.m_total_count  = _rhs.m_total_count
-    #    self.m_drain_time   = _rhs.m_drain_time
-    #    self.m_prop_mode    = _rhs.self.m_prop_mode
-    #  endfunction
-    #
-    #endclass
+
+    def do_copy(self, rhs):
+        _rhs = rhs
+        # $cast(_rhs, rhs)
+        self.m_source_count = _rhs.m_source_count
+        self.m_total_count  = _rhs.m_total_count
+        self.m_drain_time   = _rhs.m_drain_time
+        self.m_prop_mode    = _rhs.self.m_prop_mode
+
 
 #// TODO: change to plusarg
 #//`define UVM_DEFAULT_TIMEOUT 9200s
@@ -1159,190 +1228,6 @@ class UVMObjection(UVMReportObject):
     #    end
     #  endfunction
     #
-    #
-    #`ifndef UVM_NO_DEPRECATED
-    #  // m_do_stop_all
-    #  // -------------
-    #
-    #  task m_do_stop_all(uvm_component comp)
-    #
-    #    string name
-    #
-    #    // we use an external traversal to ensure all forks are
-    #    // made from a single threaad.
-    #    if (comp.get_first_child(name))
-    #      do begin
-    #        m_do_stop_all(comp.get_child(name))
-    #      end
-    #      while (comp.get_next_child(name))
-    #
-    #    if (comp.enable_stop_interrupt) begin
-    #      m_n_stop_threads++
-    #      fork begin
-    #        comp.stop_phase(run_ph)
-    #        m_n_stop_threads--
-    #      end
-    #      join_none
-    #    end
-    #  endtask
-    #
-    #
-    #  // Function- stop_request DEPRECATED
-    #  //
-    #  // Calling this function triggers the process of shutting down the currently
-    #  // running task-based phase. This process involves calling all components'
-    #  // stop tasks for those components whose enable_stop_interrupt bit is set.
-    #  // Once all stop tasks return, or once the optional global_stop_timeout
-    #  // expires, all components' kill method is called, effectively ending the
-    #  // current phase. The uvm_top will then begin execution of the next phase,
-    #  // if any.
-    #
-    #  function void stop_request()
-    #    `uvm_info_context("STOP_REQ",
-    #                      "Stop-request called. Waiting for all-dropped on uvm_test_done",
-    #                      UVM_FULL,self.m_top)
-    #    fork
-    #      m_stop_request()
-    #    join_none
-    #  endfunction
-    #
-    #  task m_stop_request()
-    #    raise_objection(self.m_top,"stop_request called; raising test_done objection")
-    #    uvm_wait_for_nba_region()
-    #    drop_objection(self.m_top,"stop_request called; dropping test_done objection")
-    #  endtask
-    #
-    #
-    #  // Variable- stop_timeout DEPRECATED
-    #  //
-    #  // These set watchdog timers for task-based phases and stop tasks. You cannot
-    #  // disable the timeouts. When set to 0, a timeout of the maximum time possible
-    #  // is applied. A timeout at this value usually indicates a problem with your
-    #  // testbench. You should lower the timeout to prevent "never-ending"
-    #  // simulations.
-    #
-    #  time stop_timeout = 0
-    #
-    #
-    #  // Task- all_dropped DEPRECATED
-    #  //
-    #  // This callback is called when the given ~object's~ objection count reaches
-    #  // zero; if the ~object~ is the implicit top-level, <uvm_root> then it means
-    #  // there are no more objections raised for the ~uvm_test_done~ objection.
-    #  // Thus, after calling <uvm_objection::all_dropped>, this method will call
-    #  // <global_stop_request> to stop the current task-based phase (e.g. run).
-    #
-    #  virtual task all_dropped (uvm_object obj,
-    #                            uvm_object source_obj,
-    #                            string description,
-    #                            int count)
-    #    if (obj != self.m_top) begin
-    #      super.all_dropped(obj,source_obj,description,count)
-    #      return
-    #    end
-    #
-    #    self.m_top.all_dropped(this, source_obj, description, count)
-    #
-    #    // All stop tasks are forked from a single thread within a 'guard' process
-    #    // so 'disable fork' can be used.
-    #
-    #    if(m_cleared == 0) begin
-    #      `uvm_info_context("TEST_DONE",
-    #          "All end-of-test objections have been dropped. Calling stop tasks",
-    #          UVM_FULL,self.m_top)
-    #      fork begin // guard
-    #        fork
-    #          begin
-    #            m_executing_stop_processes = 1
-    #            m_do_stop_all(self.m_top)
-    #            wait (m_n_stop_threads == 0)
-    #            m_executing_stop_processes = 0
-    #          end
-    #          begin
-    #            if (stop_timeout == 0)
-    #              wait(stop_timeout != 0)
-    #            `uvm_delay(stop_timeout)
-    #            `uvm_error("STOP_TIMEOUT",
-    #              {$sformatf("Stop-task timeout of %0t expired. ", stop_timeout),
-    #                 "'run' phase ready to proceed to extract phase"})
-    #          end
-    #        join_any
-    #        disable fork
-    #      end
-    #      join // guard
-    #
-    #      `uvm_info_context("TEST_DONE", {"'run' phase is ready ",
-    #                        "to proceed to the 'extract' phase"}, UVM_LOW,self.m_top)
-    #
-    #    end
-    #
-    #    if (self.m_events.exists(obj))
-    #      ->self.m_events[obj].all_dropped
-    #    self.m_top_all_dropped = 1
-    #
-    #  endtask
-    #
-    #
-    #  // Function- raise_objection DEPRECATED
-    #  //
-    #  // Calls <uvm_objection::raise_objection> after calling <qualify>.
-    #  // If the ~object~ is not provided or is ~null~, then the implicit top-level
-    #  // component, ~uvm_top~, is chosen.
-    #
-    #  virtual function void raise_objection (uvm_object obj=null,
-    #                                         string description="",
-    #                                         int count=1)
-    #    if(obj==null)
-    #      obj=self.m_top
-    #    else
-    #      qualify(obj, 1, description)
-    #
-    #    if (m_executing_stop_processes) begin
-    #      string desc = description == "" ? "" : {"(\"", description, "\") "}
-    #      `uvm_warning("ILLRAISE", {"The uvm_test_done objection was ",
-    #        "raised ", desc, "during processing of a stop_request, i.e. stop ",
-    #        "task execution. The objection is ignored by the stop process"})
-    #        return
-    #    end
-    #
-    #    super.raise_objection(obj,description,count)
-    #
-    #  endfunction
-    #
-    #
-    #  // Function- drop_objection DEPRECATED
-    #  //
-    #  // Calls <uvm_objection::drop_objection> after calling <qualify>.
-    #  // If the ~object~ is not provided or is ~null~, then the implicit top-level
-    #  // component, ~uvm_top~, is chosen.
-    #
-    #  virtual function void drop_objection (uvm_object obj=null,
-    #                                        string description="",
-    #                                        int count=1)
-    #    if(obj==null)
-    #      obj=self.m_top
-    #    else
-    #      qualify(obj, 0, description)
-    #    super.drop_objection(obj,description,count)
-    #  endfunction
-    #
-    #
-    #  // Task- force_stop DEPRECATED
-    #  //
-    #  // Forces the propagation of the all_dropped() callback, even if there are still
-    #  // outstanding objections. The net effect of this action is to forcibly end
-    #  // the current phase.
-    #
-    #  virtual task force_stop(uvm_object obj=null)
-    #    uvm_report_warning("FORCE_STOP",{"Object '",
-    #       (obj!=null?obj.get_name():"<unknown>"),"' called force_stop"})
-    #    m_cleared = 1
-    #    all_dropped(self.m_top,obj,"force_stop() called",1)
-    #    clear(obj)
-    #  endtask
-    #`endif
-    #
-    #
     #  // Below are basic data operations needed for all uvm_objects
     #  // for factory registration, printing, comparing, etc.
     #
@@ -1397,72 +1282,13 @@ class UVMObjectionContextObject:
         self.description = ""
         self.count = 0
         self.objection = UVMObjection()
-        # self.objection = None
-    #endclass
+
 
 #// Typedef - Exists for backwards compat
 #typedef uvm_objection uvm_callbacks_objection
-#
-#//------------------------------------------------------------------------------
-#//
-#// Class: uvm_objection_callback
-#//
-#//------------------------------------------------------------------------------
-#// The uvm_objection is the callback type that defines the callback
-#// implementations for an objection callback. A user uses the callback
-#// type uvm_objection_cbs_t to add callbacks to specific objections.
-#//
-#// For example:
-#//
-#//| class my_objection_cb extends uvm_objection_callback
-#//|   function new(string name)
-#//|     super.new(name)
-#//|   endfunction
-#//|
-#//|   virtual function void raised (uvm_objection objection, uvm_object obj,
-#//|       uvm_object source_obj, string description, int count)
-#//|       `uvm_info("RAISED","%0t: Objection %s: Raised for %s", $time, objection.get_name(),
-#//|       obj.get_full_name())
-#//|   endfunction
-#//| endclass
-#//| ...
-#//| initial begin
-#//|   my_objection_cb cb = new("cb")
-#//|   uvm_objection_cbs_t::add(null, cb); //typewide callback
-#//| end
-#
-#
-#class uvm_objection_callback extends uvm_callback
-#  function new(string name)
-#    super.new(name)
-#  endfunction
-#
-#  // Function: raised
-#  //
-#  // Objection raised callback function. Called by <uvm_objection::raised>.
-#
-#  virtual function void raised (uvm_objection objection, uvm_object obj,
-#      uvm_object source_obj, string description, int count)
-#  endfunction
-#
-#  // Function: dropped
-#  //
-#  // Objection dropped callback function. Called by <uvm_objection::dropped>.
-#
-#  virtual function void dropped (uvm_objection objection, uvm_object obj,
-#      uvm_object source_obj, string description, int count)
-#  endfunction
-#
-#  // Function: all_dropped
-#  //
-#  // Objection all_dropped callback function. Called by <uvm_objection::all_dropped>.
-#
-#  virtual task all_dropped (uvm_objection objection, uvm_object obj,
-#      uvm_object source_obj, string description, int count)
-#  endtask
-#
-#endclass
-#
-#
+
+
+
+
 #`endif
 #
