@@ -350,6 +350,14 @@ class UVMPhase(UVMObject):
     #  // domain.
     #  //
     #  extern function uvm_phase find_by_name(string name, bit stay_in_scope=1)
+    def find_by_name(self, name: str, stay_in_scope=1):
+        # TODO: full search
+        if self.get_name() == name:
+            return self
+        find_by_name = self.m_find_predecessor_by_name(name,stay_in_scope,self)
+        if find_by_name is None:
+            find_by_name = self.m_find_successor_by_name(name,stay_in_scope,self)
+        return find_by_name
 
     #  Function: find
     #
@@ -754,12 +762,12 @@ class UVMPhase(UVMObject):
         if obj is not None:
             m_obj_name = obj.get_full_name()
 
-        if ((action == "raise") or (action == "drop")):
+        if action == "raise" or action == "drop":
             if count != 1:
                 m_action = "{} {} objections".format(action, count)
             else:
                 m_action = "{} an objection".format(action)
-        elif (action == "get_objection_count"):
+        elif action == "get_objection_count":
             m_action = "call get_objection_count"
 
         if self.get_phase_type() == UVM_PHASE_IMP:
@@ -774,8 +782,10 @@ class UVMPhase(UVMObject):
     #  //
     #  // Return the <uvm_objection> that gates the termination of the phase.
     #  //
-    #  function uvm_objection get_objection(); return this.phase_done; endfunction
-    #
+    def get_objection(self):
+        return self.phase_done
+
+
     #  // Function: raise_objection
     #  //
     #  // Raise an objection to ending this phase
@@ -805,7 +815,12 @@ class UVMPhase(UVMObject):
     #  // The drop is expected to be matched with an earlier raise.
     #  //
     def drop_objection(self, obj, description="", count=1):
+        if self.get_name() == 'reset':
+            print("EEE object dropping reset obj now " + obj.get_name())
         if self.phase_done is not None:
+            if self.get_name() == 'reset':
+                print("Dropping reset objection\n" +
+                        self.phase_done.convert2string())
             self.phase_done.drop_objection(obj,description,count)
         else:
             self.m_report_null_objection(obj, description, count, "drop")
@@ -994,6 +1009,8 @@ class UVMPhase(UVMObject):
     #  // is in progress. Valid for use during the phase_ended() callback
     #  //
     #  extern function uvm_phase get_jump_target()
+    def get_jump_target(self):
+        return self.m_jump_phase
 
 
     #// m_find_predecessor
@@ -1047,15 +1064,44 @@ class UVMPhase(UVMObject):
             else:
                 orig = orig_phase
             if (not stay_in_scope or
-                (succ.get_schedule() == orig.get_schedule()) or
-                (succ.get_domain() == orig.get_domain())):
+                    (succ.get_schedule() == orig.get_schedule()) or
+                    (succ.get_domain() == orig.get_domain())):
                 found = succ.m_find_successor(phase,stay_in_scope,orig)
                 return found
         return None
 
 
     #  extern function uvm_phase m_find_predecessor_by_name(string name, bit stay_in_scope=1, uvm_phase orig_phase=null)
+    def m_find_predecessor_by_name(self, name, stay_in_scope=1, orig_phase=None):
+        found = None
+        if self.get_name() == name:
+            return self
+        for pred in self.m_predecessors:
+            orig = self if (orig_phase is None) else orig_phase
+            if (not stay_in_scope or
+                    (pred.get_schedule() == orig.get_schedule()) or
+                    (pred.get_domain() == orig.get_domain())):
+                found = pred.m_find_predecessor_by_name(name,stay_in_scope,orig)
+                if found is not None:
+                    return found
+        return None
+
     #  extern function uvm_phase m_find_successor_by_name(string name, bit stay_in_scope=1, uvm_phase orig_phase=null)
+    def m_find_successor_by_name(self, name, stay_in_scope=1, orig_phase=None):
+        found = None
+        if self.get_name() == name:
+            return self
+        for succ in self.m_successors:
+            orig = self if (orig_phase is None) else orig_phase
+            if (not stay_in_scope or
+                    (succ.get_schedule() == orig.get_schedule()) or
+                    (succ.get_domain() == orig.get_domain())):
+                found = succ.m_find_successor_by_name(name,stay_in_scope,orig)
+                if found is not None:
+                    return found
+        return None
+
+
     #  extern function void m_print_successors()
 
     #
@@ -1242,7 +1288,6 @@ class UVMPhase(UVMObject):
             uvm_debug(self, 'execute_phase', 'PHASE_DONE_REACHED - returning now')
             return
 
-        #state_chg = uvm_phase_state_change::type_id::create(get_name())
         state_chg = UVMPhaseStateChange(self.get_name())
         state_chg.m_phase      = self
         state_chg.m_jump_to    = None
@@ -1746,67 +1791,9 @@ class UVMPhaseCb(UVMCallback):
     #endfunction
     #
     #
-    #// m_find_predecessor_by_name
-    #// --------------------------
-    #
-    #function uvm_phase uvm_phase::m_find_predecessor_by_name(string name, bit stay_in_scope=1, uvm_phase orig_phase=null)
-    #  uvm_phase found
-    #  //$display("  FIND PRED node '",name,"' - checking against ",get_name()," (",self.m_phase_type.name()," id=",$sformatf("%0d",get_inst_id()),(m_imp==null)?"":{"/",$sformatf("%0d",m_imp.get_inst_id())},")")
-    #  if (get_name() == name)
-    #    return this
-    #  foreach (self.m_predecessors[pred]) begin
-    #    uvm_phase orig
-    #    orig = (orig_phase==null) ? this : orig_phase
-    #    if (!stay_in_scope ||
-    #        (pred.get_schedule() == orig.get_schedule()) ||
-    #        (pred.get_domain() == orig.get_domain())) begin
-    #      found = pred.m_find_predecessor_by_name(name,stay_in_scope,orig)
-    #      if (found != null)
-    #        return found
-    #    end
-    #  end
-    #  return null
-    #endfunction
-    #
-    #
-    #// m_find_successor_by_name
-    #// ------------------------
-    #
-    #function uvm_phase uvm_phase::m_find_successor_by_name(string name, bit stay_in_scope=1, uvm_phase orig_phase=null)
-    #  uvm_phase found
-    #  //$display("  FIND SUCC node '",name,"' - checking against ",get_name()," (",self.m_phase_type.name()," id=",$sformatf("%0d",get_inst_id()),(m_imp==null)?"":{"/",$sformatf("%0d",m_imp.get_inst_id())},")")
-    #  if (get_name() == name)
-    #    return this
-    #  foreach (self.m_successors[succ]) begin
-    #    uvm_phase orig
-    #    orig = (orig_phase==null) ? this : orig_phase
-    #    if (!stay_in_scope ||
-    #        (succ.get_schedule() == orig.get_schedule()) ||
-    #        (succ.get_domain() == orig.get_domain())) begin
-    #      found = succ.m_find_successor_by_name(name,stay_in_scope,orig)
-    #      if (found != null)
-    #        return found
-    #    end
-    #  end
-    #  return null
-    #endfunction
-    #
-    #// find_by_name
-    #// ------------
-    #
-    #function uvm_phase uvm_phase::find_by_name(string name, bit stay_in_scope=1)
-    #  // TBD full search
-    #  //$display({"\nFIND node named '",name,"' within ",get_name()," (scope ",self.m_phase_type.name(),")", (stay_in_scope) ? " staying within scope" : ""})
-    #  if (get_name() == name)
-    #    return this
-    #  find_by_name = m_find_predecessor_by_name(name,stay_in_scope,this)
-    #  if (find_by_name == null)
-    #    find_by_name = m_find_successor_by_name(name,stay_in_scope,this)
-    #endfunction
-    #
-    #
-    #
-    #
+
+
+
     #function void uvm_phase::get_adjacent_predecessor_nodes(ref uvm_phase pred[])
     #   bit done
     #   bit predecessors[uvm_phase]
@@ -2045,17 +2032,6 @@ class UVMPhaseCb(UVMCallback):
     #endfunction
     #
     #
-    #// get_jump_target
-    #// ---------------
-    #
-    #function uvm_phase uvm_phase::get_jump_target()
-    #  return m_jump_phase
-    #endfunction
-    #
-    #
-    #
-    #
-    #
 
     #
     #// kill
